"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createComputeBudgetIx = exports.sendAndConfirmSignedTransaction = exports.prepareSignedTransaction = exports.sendTransaction = void 0;
const nodewallet_1 = __importDefault(require("@coral-xyz/anchor/dist/cjs/nodewallet"));
const buffer_layout_1 = require("@solana/buffer-layout");
const web3_js_1 = require("@solana/web3.js");
const constants_1 = require("../constants");
async function sendTransaction(provider, ixs, alts, opts = {}) {
    const connection = provider.connection;
    const latestBlockhash = opts.latestBlockhash ??
        (await connection.getLatestBlockhash(opts.preflightCommitment ??
            provider.opts.preflightCommitment ??
            'finalized'));
    const vtx = await prepareSignedTransaction(provider, ixs, alts, latestBlockhash, opts);
    if (opts.multipleProviders) {
        const providers = [provider, ...opts.multipleProviders];
        return Promise.race(providers.map((p) => sendAndConfirmSignedTransaction(p, vtx, latestBlockhash, opts)));
    }
    else {
        return sendAndConfirmSignedTransaction(provider, vtx, latestBlockhash, opts);
    }
}
exports.sendTransaction = sendTransaction;
async function prepareSignedTransaction(provider, ixs, alts, latestBlockhash, opts = {}) {
    const payer = provider.wallet;
    //
    // setComputeUnitLimit, hard code to a higher minimum, this is needed so that we dont fail simple UI interactions
    //
    // https://github.com/solana-labs/solana-web3.js/blob/master/packages/library-legacy/src/programs/compute-budget.ts#L202
    const computeUnitLimitIxFound = ixs.some((ix) => ix.programId.equals(constants_1.COMPUTE_BUDGET_PROGRAM_ID) &&
        (0, buffer_layout_1.u8)().decode(ix.data.subarray(0, 1)) == 2);
    if (!computeUnitLimitIxFound) {
        const totalUserIntendedIxs = ixs.filter((ix) => !ix.programId.equals(constants_1.COMPUTE_BUDGET_PROGRAM_ID)).length;
        const requestCu = Math.min(totalUserIntendedIxs * 300_000, 1_600_000);
        ixs = [
            web3_js_1.ComputeBudgetProgram.setComputeUnitLimit({
                units: requestCu,
            }),
            ...ixs,
        ];
    }
    //
    // setComputeUnitPrice
    //
    if (opts.prioritizationFee) {
        ixs = [(0, exports.createComputeBudgetIx)(opts.prioritizationFee), ...ixs];
    }
    const message = web3_js_1.MessageV0.compile({
        payerKey: provider.wallet.publicKey,
        instructions: ixs,
        recentBlockhash: latestBlockhash.blockhash,
        addressLookupTableAccounts: alts,
    });
    let vtx = new web3_js_1.VersionedTransaction(message);
    if (opts?.additionalSigners?.length) {
        vtx.sign([...opts?.additionalSigners]);
    }
    if (typeof payer.signTransaction === 'function' &&
        !(payer instanceof nodewallet_1.default || payer.constructor.name == 'NodeWallet')) {
        vtx = (await payer.signTransaction(vtx));
    }
    else {
        // Maybe this path is only correct for NodeWallet?
        vtx.sign([payer.payer]);
    }
    return vtx;
}
exports.prepareSignedTransaction = prepareSignedTransaction;
async function sendAndConfirmSignedTransaction(provider, signedTransaction, latestBlockhash, opts = {}) {
    const connection = provider.connection;
    const signature = await connection.sendRawTransaction(signedTransaction.serialize(), {
        skipPreflight: true, // mergedOpts.skipPreflight,
    });
    if (opts.postSendTxCallback) {
        try {
            opts.postSendTxCallback({ txid: signature });
        }
        catch (e) {
            console.warn(`postSendTxCallback error ${e}`);
        }
    }
    const txConfirmationCommitment = opts.txConfirmationCommitment ?? 'processed';
    let status;
    if (latestBlockhash.blockhash != null &&
        latestBlockhash.lastValidBlockHeight != null) {
        status = await connection.confirmTransaction({
            signature: signature,
            blockhash: latestBlockhash.blockhash,
            lastValidBlockHeight: latestBlockhash.lastValidBlockHeight,
        }, txConfirmationCommitment);
    }
    else {
        status = await connection.confirmTransaction(signature, txConfirmationCommitment);
    }
    const signatureResult = status.value;
    if (signatureResult.err) {
        console.warn('Tx status: ', status);
        throw new MangoError({
            txid: signature,
            message: `${JSON.stringify(status)}`,
        });
    }
    return { signature, slot: status.context.slot, ...signatureResult };
}
exports.sendAndConfirmSignedTransaction = sendAndConfirmSignedTransaction;
const createComputeBudgetIx = (microLamports) => {
    const computeBudgetIx = web3_js_1.ComputeBudgetProgram.setComputeUnitPrice({
        microLamports,
    });
    return computeBudgetIx;
};
exports.createComputeBudgetIx = createComputeBudgetIx;
class MangoError extends Error {
    message;
    txid;
    constructor({ txid, message }) {
        super();
        this.message = message;
        this.txid = txid;
    }
}
